@@ -30,6 +30,7 @@
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-arj.h"
 
@@ -345,6 +346,8 @@ fr_command_arj_class_init (FRCommandArjClass *class)
 static void 
 fr_command_arj_init (FRCommand *comm)
 {
+	comm->file_type = FR_FILE_TYPE_ARJ;
+
 	comm->propCanModify                = TRUE;
 	comm->propAddCanUpdate             = TRUE; 
 	comm->propAddCanReplace            = TRUE; 
