@@ -77,6 +77,7 @@ typedef struct {
 	GtkListStore *   list_store;
 	GtkWidget *      toolbar;
 	GtkWidget *      statusbar;
+	GtkWidget *      progress_bar;
 	GtkWidget *      location_bar;
 	GtkWidget *      location_entry;
 	GtkWidget *      location_label;
@@ -98,6 +99,7 @@ typedef struct {
 	GtkWidget *      down_arrows[5];
 
 	FRArchive *      archive;
+	FRAction         current_action;
 	gboolean         archive_present;
 	gboolean         archive_new;        /* A new archive has been created
 					      * but it doesn't contain any 
@@ -196,11 +198,15 @@ typedef struct {
 	/* progress dialog data */
 
 	GtkWidget *progress_dialog;
+	GtkWidget *pd_action;
+	GtkWidget *pd_archive;
 	GtkWidget *pd_message;
 	GtkWidget *pd_progress_bar;
 	gboolean   progress_pulse;
 	guint      progress_timeout;  /* Timeout to display the progress dialog. */
 	guint      hide_progress_timeout;  /* Timeout to hide the progress dialog. */
+	FRAction   pd_last_action;
+	char      *pd_last_archive;
 
 	/* batch mode data */
 
@@ -266,6 +272,14 @@ void       window_archive_add_directory     (FRWindow      *window,
 					     FRCompression  compression);
 
 void       window_archive_add_items         (FRWindow      *window,
+					     GList         *dir_list,
+					     const char    *base_dir,
+					     const char    *dest_dir,
+					     gboolean       update,
+					     const char    *password,
+					     FRCompression  compression);
+
+void       window_archive_add_dropped_items (FRWindow      *window,
 					     GList         *item_list,
 					     gboolean       update);
 
@@ -313,6 +327,10 @@ GList *    window_get_file_list_selection   (FRWindow    *window,
 					     gboolean     recursive,
 					     gboolean    *has_dirs);
 
+GList *    window_get_file_list_from_path_list (FRWindow *window,
+						GList    *path_list,
+						gboolean *has_dirs);
+
 GList *    window_get_file_list_pattern     (FRWindow    *window,
 					     const char  *pattern);
 
@@ -414,4 +432,8 @@ void       window_archive__quit               (FRWindow      *window);
 
 void       window_convert_data_free           (FRWindow *window);
 
+gboolean   fr_window_file_list_drag_data_get  (FRWindow         *window,
+					       GList            *path_list,
+					       GtkSelectionData *selection_data);
+
 #endif /* FR_WINDOW_H */
